#ifndef POOL_H
#define POOL_H

#define MAX_POOL_PAGE_SIZE PAGESIZE

typedef enum _MEMORY_POOL_FLAGS {
	POOL_FIXED = 1,
	POOL_NON_PAGEABLE = 2,
	POOL_LOCAL = 4,
	POOL_STATIC = 8,
	POOL_SHRINKABLE = 16
} MEMORY_POOL_FLAGS;

typedef struct _MEMORY_POOL_HEADER {
	struct _MEMORY_POOL_HEADER *Next;
	u32 Flags;
	u32 Internal;
} MEMORY_POOL_HEADER;

typedef struct _MEMORY_POOL {
	u32 TypeFlags;				/* flags see MEMORY_POOL_FLAGS */
	u32 ItemSize;				/* fixed size of each item */
	u32 PageSize;				/* size of each pool page */
	u32 ItemsPerPage;
	u32 Size;					/* entire pool memory size inclusive headers */
	u32 PageCount;				/* number of currently allocated pool pages */
	s32 AvlCount;				/* available number of free items */
	u32 MaxCount;				/* max. number of items pool can store */
	u32 MaxSize;				/* max memory pool size in Bytes */
	u32 ExpandThreshold;		/* threshold after MmResizePool will be called */
	u32 ShrinkThreshold;		/* percentage of free pages afterward pool will be shrinked */
	s32 Fragmentation;
	MEMORY_POOL_HEADER *Header;
	void *NextFree;
	u32 (*Alloc) (u32 Pid, void **Base, u32 Size, u32 Flags);
	u32 (*Free) (u32 Pid, void **Base, u32 Flags);
} MEMORY_POOL;

u32 MmCreatePool(MEMORY_POOL *Pool, void* Data, u32 PoolItemSize, MEMORY_POOL_FLAGS TypeFlags);
u32 MmDestroyPool(MEMORY_POOL *Pool);
u32 MmExpandPool(MEMORY_POOL *Pool);
u32 MmShrinkPool(MEMORY_POOL *Pool);
u32 MmAllocPoolItem(MEMORY_POOL *Pool, void** Data);
u32 MmGetPoolFragmentation(MEMORY_POOL* Pool, u32 *Percentage);
u32 MmSetPoolExpandThreshold(MEMORY_POOL *Pool, u32 Threshold);
u32 MmSetPoolShrinkThreshold(MEMORY_POOL *Pool, u32 Threshold);
u32 MmSetPoolExpandSize(MEMORY_POOL *Pool, u32 Size);
u32 MmFreePoolItem(MEMORY_POOL *Pool, void* Data);

#endif
